@@ -1,93 +1,93 @@
-GOOF----LE-4-2.0Ƅ      ] 4   h-      ] g  guile	 	g  define-module*	 	 	g  web	g  client	 		g  filenameS	
+GOOF----LE-4-2.0      ] 4       h      ] g  guile	 	g  define-module*	 	 	g  web	g  client	 		g  filenameS	
 f  web/client.scm	g  importsS	g  rnrs	
 g  bytevectors	
- 	 	g  ice-9	g  binary-ports	 	 	g  iconv	 	 	g  rdelim	 	 	g  request	 	 	g  response	 	 	 g  uri	!  	"! 	#g  srfi	$g  srfi-1	%#$ 	&% 	'"& 	(g  exportsS	)g  open-socket-for-uri	*g  http-get	+g  	http-get*	,g  	http-head	-g  	http-post	.g  http-put	/g  http-delete	0g  
-http-trace	1g  http-options	2)*+,-./01 		3g  set-current-module	43 	53 	6g  string?	7g  string->uri	8g  error	9f  Invalid URI	:g  <uri>	;!: 	<!: 	=g  
-ensure-uri	>g  srfi-9	?#> 	@g  throw-bad-struct	A?@ 	B?@ 	Cg  uri-port	Dg  delete-duplicates	Eg  getaddrinfo	Fg  uri-host	Gg  number->string	Hg  symbol->string	Ig  
-uri-scheme	Jg  AI_NUMERICSERV	Kg  
-addrinfo:addr	Lg  %default-port-encoding	Mg  socket	Ng  addrinfo:fam	Og  SOCK_STREAM	Pg  
-IPPROTO_IP	Qg  catch	Rg  system-error	Sg  connect	Tg  setvbuf	Ug  _IOFBF	Vg  
-setsockopt	Wg  
-SOL_SOCKET	Xg  	SO_RCVBUF	Yg  close	Zg  throw	[g  
-build-request	\g  	<request>	]\ 	^\ 	_g  request-uri	`g  versionS	ag  request-version	bg  headersS	cg  
-assoc-set!	dg  	copy-tree	eg  request-headers	fg  portS	gg  request-port	hg  extend-request	ig  request-content-type	jg  
-text/plain	kj 	lg  assq-ref	mg  charset	ng  sanitize-request	og  content-type	pg  append	qp 	rp 	sf  utf-8	tg  string->bytevector	ug  
-procedure?	vg  call-with-encoded-output-string	wg  bytevector?	xg  request-content-length	yg  bytevector-length	zf  bad content-length	{g  content-length	|f  unexpected body type	}f  content-length, but no body	~g  assq	g  transfer-encoding f  *transfer-encoding not allowed with no body g  vu8
-	      j    g  response-content-length g  response-content-type g  text-content-type? g  bytevector->string f  
-iso-8859-1 g  decode-response-body g  bodyS  f	 g  methodS 	 `	 g  keep-alive?S 	 b	 g  decode-body?S 	 g  
-streaming?S 	 g  requestS 		  	 f  GET  g  
-connection Y  g  
-write-request g  write-request-body g  force-output g  
-read-response g  request-method f  HEAD g  
-close-port g  response-body-port g  decode?S g  read-response-body `	 	 g  
+ 	 	g  ice-9	g  binary-ports	 	 	g  iconv	 	 	g  rdelim	 	 	g  request	 	 	g  response	 	 	 g  uri	!  	"! 	#g  http	$# 	%$ 	&g  srfi	'g  srfi-1	(&' 	)( 	*g  srfi-9	+&* 	,+ 	-g  gnu	.&*- 	/. 	0"%),/ 	1g  exportsS	2g  current-http-proxy	3g  open-socket-for-uri	4g  http-get	5g  	http-get*	6g  	http-head	7g  	http-post	8g  http-put	9g  http-delete	:g  
+http-trace	;g  http-options	<23456789:; 
+	=g  set-current-module	>= 	?= 	@g  make-parameter	Ag  getenv	Bf  
+http_proxy	Cf   	Dg  string?	Eg  string->uri	Fg  error	Gf  Invalid URI	Hg  <uri>	I!H 	J!H 	Kg  
+ensure-uri	Lg  throw-bad-struct	M+L 	N+L 	Og  uri-port	Pg  delete-duplicates	Qg  getaddrinfo	Rg  uri-host	Sg  number->string	Tg  symbol->string	Ug  
+uri-scheme	Vg  AI_NUMERICSERV	Wg  
+addrinfo:addr	Xg  %default-port-encoding	Yg  socket	Zg  addrinfo:fam	[g  SOCK_STREAM	\g  
+IPPROTO_IP	]g  catch	^g  system-error	_g  connect	`g  setvbuf	ag  _IOFBF	bg  
+setsockopt	cg  
+SOL_SOCKET	dg  	SO_RCVBUF	eg  set-http-proxy-port?!	fg  close	gg  throw	hg  	<request>	ih 	jh 	kg  
+assoc-set!	lg  	copy-tree	mg  %%<request>-set-fields	ng  extend-request	og  request-content-type	pg  
+text/plain	qp 	rg  assq-ref	sg  charset	tg  sanitize-request	ug  content-type	vg  append	wv 	xv 	yf  utf-8	zg  string->bytevector	{g  
+procedure?	|g  call-with-encoded-output-string	}g  bytevector?	~g  request-content-length	g  bytevector-length f  bad content-length g  content-length f  unexpected body type f  content-length, but no body g  assq g  transfer-encoding g  request-headers f  *transfer-encoding not allowed with no body g  vu8
+	  j    g  response-content-length g  response-content-type g  text-content-type? g  bytevector->string f  
+iso-8859-1 g  decode-response-body g  bodyS  g  portS 	 g  methodS 	 g  versionS 	 g  keep-alive?S 	 g  headersS 	 g  decode-body?S 	 g  
+streaming?S 	 g  requestS 		  	 g  GET  g  
+build-request g  
+connection f  g  
+write-request g  write-request-body g  force-output g  request-port g  
+read-response g  request-method g  HEAD g  
+close-port g  response-body-port g  decode?S g  read-response-body 	 	 g  
 extra-headersS 	   g  issue-deprecation-warning f  HThe #:extra-headers argument to http-get has been renamed to #:headers.  f  Please update your code.   f  "`http-get*' has been deprecated.   f  BInstead, use `http-get' with the #:streaming? #t keyword argument. g  make-syntax-transformer     g  define-http-verb g  macro g  $sc-dispatch     g  any   g  
-syntax-object g  define* g  top   g  ribcage g  dummy g  	http-verb g  method g  doc   g  m-AcABDQBYQg5jbMyxoYWabA-5176     f  l-AcABDQBYQg5jbMyxoYWabA-5181 f  l-AcABDQBYQg5jbMyxoYWabA-5182 f  l-AcABDQBYQg5jbMyxoYWabA-5183 f  l-AcABDQBYQg5jbMyxoYWabA-5184       g  x     f  l-AcABDQBYQg5jbMyxoYWabA-5178       g  hygiene        g  keyS   g  body       g  port   )      g  version   g  quote     錤     g  keep-alive?     g  headers       g  decode-body?       g  
-streaming?      	       f  `  b     g  syntax-violation  	f  -source expression failed to match any pattern
-b			
-
- f  POSTf  PUTf  DELETEf  TRACEf  OPTIONSC 5      ho     ]4	
-'(25 45 >  "  G   6789<      h@      ]4 5$   6"   6 $   &   C""             g  
+syntax-object g  define* g  top   g  ribcage g  dummy g  	http-verb g  method g  doc   g  m-CCSxwggPgrtKFRq9kkHjhx-5361     f  l-CCSxwggPgrtKFRq9kkHjhx-5366 f  l-CCSxwggPgrtKFRq9kkHjhx-5367 f  l-CCSxwggPgrtKFRq9kkHjhx-5368 f  l-CCSxwggPgrtKFRq9kkHjhx-5369       g  x     f  l-CCSxwggPgrtKFRq9kkHjhx-5363       g  hygiene        g  keyS   g  body       g  port   3      g  version   g  quote          g  keep-alive?     g  headers       g  decode-body?    g  
+streaming?   	 	 
+   
+    
+ g  syntax-violation  f  -source expression failed to match any pattern			 g  POSTg  PUTg  DELETEg  TRACEg  OPTIONSC 5 ho  -  ]4	
+01<5 4? >  "  G   4@i4AiB5  C$  "    52RDEFGJ      h@      ]4 5$   6"   6 $   &   C""             g  
 uri-or-string
 		:  g  filenamef  web/client.scm
-	4
-		6		
-	5			6			8			8				5		"	7		&	5		)	7	 		:  g  nameg  
-ensure-uri C=R=<BCDEFGHIJK     h   |   ]4 545C      t       g  ai1
+	=
+		?		
+	>			?			A			A				>		"	@		&	>		)	@	 		:  g  nameg  
+ensure-uri CKR2KJNOPQRSTUVW   h   |   ]4 545C      t       g  ai1
 		 g  ai2		  g  filenamef  web/client.scm
-	F			G		
-	G	%		G		 			   CLMNOPQRSKTUVWX   hP   y   ] 4L4L 5>  "  G  4L>  "  G  4L0 >  "  G  LC     q       g  filenamef  web/client.scm
-	O			P	
-		P			P	
-		S	
-	0	U	
-	;	U	-	@	U	
- 			K
-   CYZ       h0      -  1  3 4L>  "  G  L (   @LL 6 ~       g  args
+	P			Q		
+	Q	%		Q		 			   CXYZ[\]^_W`abcde 	hh      ] 4L4L5>  "  G  4L>  "  G  4L0 >  "  G  M $  4L>  "  G  "   LC       g  filenamef  web/client.scm
+	Y			Z	
+		Z			Z	
+		]	
+	0	_	
+	;	_	-	@	_	
+	N	a	
+	O	a	 		h
+   Cfg       h0      -  1  3 4L>  "  G  L (   @LL 6 ~       g  args
 			/  g  filenamef  web/client.scm
-	W		
-	Y	
-		Z		"	Z	
-	(	[		-	\		/	\	 			/
+	c		
+	e	
+		f		"	f	
+	(	g		-	h		/	h	 			/
 
 
    C 
-      h@      ] Y4455ZO 	 L O 6              g  	addresses
-		9 g  ai		9 g  s			9  g  filenamef  web/client.scm
-	I			J			J		
-	M			M			M			J		#	N	
-	9	N	 
-		9  g  nameg  loop C     h     ]HH4 5KJ&  	J	"  	4J544J&  	J	"  	4J5$  45"  4	J&  J
-"  	4J
-55$  "  
-55K
-O Q J6            g  
+      h@      ] Y4455ZLO 	 L O 6            g  	addresses
+		; g  ai		; g  s			;  g  filenamef  web/client.scm
+	S			T			T		
+	W			W			W			T		#	X	
+	;	X	 
+		;  g  nameg  loop C     h     ]!HHH45 K4J$  J"   5KJ&  	J	"  	4J544J&  	J	"  	4J5$  4	5"  4
+J&  J
+"  	4J55$  "  
+5
+5KO Q J6           g  
 uri-or-string
-	  g  uri	  g  	addresses		  g  port		,  g  loop	    g  filenamef  web/client.scm
-	:
-			<			>		,	>		/	?		2	@		7	@		U	A		a	B	 	f	B	0		B	  	C	 	@	 	?	 	I	 	   g  nameg  open-socket-for-urig  
-documentationf  9Return an open input/output port for a connection to URI. C)R[^B_`abcdefgh      h   &  - 1 3 4 &   "  	4 5 &  	 	"  	4 544	 &  	 	"  	4 
-555 &  	 	"  	4 55(  C
-@      g  r
-		  g  k		  g  v			  g  
-additional			  g  r	    g  filenamef  web/client.scm
-	^
-	
-	_			_		+	_	4	F	a		I	a	&	N	a	1	h	a	&	n	a		s	c	! 	_	 	_	 	d	 	f	 		 	
-	  g  nameg  extend-request ChR6iklmnhorstuvwxy8z{|}~^Be   h    ]$ (45$  T4 545$   "  4 4	
+	  g  
+http-proxy	  g  uri		  g  	addresses		  g  port		B  g  loop	    g  filenamef  web/client.scm
+	C
+		E			F			F		%	F		*	H		B	H		E	I		H	J		M	J		k	K		w	L	 	|	L	0 	L	  	M	 	J	 	I	 	S	 	   g  nameg  open-socket-for-urig  
+documentationf  9Return an open input/output port for a connection to URI. C3RjklNmn  hh     - 1 3  &  3 
+  	44 	55 	 	 "  	4 5(  C@              g  r
+			a g  k			a g  v				a g  
+additional				a g  r		N	a  g  filenamef  web/client.scm
+	j
+		k		"	l		%	l	"	,	l	-	.	l	"	4	l		9	k		N	k		V	n		a	p	 			a	
+	  g  nameg  extend-request CnRDoqrstnuxyz{|}~FjN      h    ]$ 45$  T4 545$   "  4 4	
  554$  "  
 5645$  T4 545$   "  4 4	
  554
 $  "  
-5645$  Y4 545$  $   "  45"  
-$   "  4 5D64 5$  "
-$  "  4>  "  G  "  <4 &  	 	"  	4 55$  4>  "  G  "    D           g  request
-	 g  body	 g  type			g g  declared-charset		(	g g  type		{  g  declared-charset	   g  rlen	 & g  blen	 & g  length	7  	g  filenamef  web/client.scm
-	i
-		|			 			|		 		 	/	 		 		 		$ 	'	& 	2	( 		( 		2 		9 		? 	#	@ 		H 	)	I 		P 		Q 		[ 		c 	)	e 		g 		h 		r	|		s 		y 	(	{ 		{ 		~ 	  	'  	2  	  	  	  	  	#  	  	)  	  	  	  	  	)  	  	  		 	|	  	  	  	  	  	  	  	  	  	
- 	 	 	 	-  	* 	. 	0 	1	~	7	~	?		B 	G 	
-L 	P 	U 	b 	f 	i 	) 	 	
- 	 	 	 	 	 V		  g  nameg  sanitize-requestg  
-documentationk 0"   S   a   n   i   t   i   z   e   "       t   h   e       g   i   v   e   n       r   e   q   u   e   s   t       a   n   d       b   o   d   y   ,       e   n   s   u   r   i   n   g       t   h   a   t       t   h   e   y       a   r   e   
+5645$  K4 545$  $   "  45"  4 5D64 5$  "
+$  "  4>  "  G  "  <4 &  	 	"  	4 55$  4>  "  G  "    D   }      g  request
+	 g  body	 g  type			g g  declared-charset		(	g g  type		{  g  declared-charset	   g  rlen	  g  blen	  g  length	)  	g  filenamef  web/client.scm
+	s
+	 			 		 		 		 	/	 		 		 		$ 	'	& 	2	( 		( 		2 		9 		? 	#	@ 		H 	)	I 		P 		Q 		[ 		c 	)	e 		g 		h 		r 		s 		y 	(	{ 		{ 		~ 	  	'  	2  	  	  	  	  	#  	  	)  	  	  	  	  	)  	  	  		  	  	  	  	  	  	  	  	  	  	 	 	- 	 	  	" 	# 	) 	1 	4 	9 	
+> 	B 	G 	T 	X 	[ 	)u 	y 	
+z 	~ 	 	 	 	 T		  g  nameg  sanitize-requestg  
+documentation k 0"   S   a   n   i   t   i   z   e   "       t   h   e       g   i   v   e   n       r   e   q   u   e   s   t       a   n   d       b   o   d   y   ,       e   n   s   u   r   i   n   g       t   h   a   t       t   h   e   y       a   r   e   
    c   o   m   p   l   e   t   e       a   n   d       c   o   h   e   r   e   n   t   .           T   h   i   s       m   e   t   h   o   d       i   s       m   o   s   t       u   s   e   f   u   l       f   o   r       m   e   t   h   o   d   s       t   h   a   t       s   e   n   d   
    d   a   t   a       t   o       t   h   e       s   e   r   v   e   r   ,       l   i   k   e       P   O   S   T   ,       b   u   t       c   a   n       b   e       u   s   e   d       f   o   r       a   n   y       m   e   t   h   o   d   .           R   e   t   u   r   n   
    t   w   o       v   a   l   u   e   s   :       a       r   e   q   u   e   s   t       a   n   d       a       b   y   t   e   v   e   c   t   o   r   ,       p   o   s   s   i   b   l   y       t   h   e       s   a   m   e       o   n   e   s       t   h   a   t       w   e   r   e   
@@ -104,14 +104,14 @@ documentationk 0"   S   a   n   i   t   i   z   e   "       t   h   e       g  
    p   a   r   t   i   c   u   l   a   r       t   i   m   e   .   
    
    N   o   t   e       t   h   a   t       w   e       r   e   l   y       o   n       t   h   e       r   e   q   u   e   s   t       i   t   s   e   l   f       a   l   r   e   a   d   y       h   a   v   i   n   g       b   e   e   n       v   a   l   i   d   a   t   e   d   ,   
-   a   s       i   s       t   h   e       c   a   s   e       b   y       d   e   f   a   u   l   t       w   i   t   h       a       r   e   q   u   e   s   t       r   e   t   u   r   n   e   d       b   y       `   b   u   i   l   d   -   r   e   q   u   e   s   t   '   .    CnRwylm8z| 
-       h     ]"$  45$  {4 545"  D4 5$  245$  #45$  "  	6CC$  $  "
+   a   s       i   s       t   h   e       c   a   s   e       b   y       d   e   f   a   u   l   t       w   i   t   h       a       r   e   q   u   e   s   t       r   e   t   u   r   n   e   d       b   y       `   b   u   i   l   d   -   r   e   q   u   e   s   t   '   .    CtR}rsF 
+   h     ]"$  45$  {4 545"  D4 5$  245$  #45$  "  	6CC$  $  "
 6"
 6C             g  response
 	  g  body	  g  rlen		!  g  blen		!  g  t		0	m g  t		R	e  g  filenamef  web/client.scm
  
 	 			 		 		 		 		! 		* 		0 		9 		> 	#	@ 		D 	
-	I 	,	N 	6	P 	A	R 	,	R 	(	b 	,	g 		m 		x 		| 	  	  	  	  	 	 	  g  nameg  decode-response-body CR)[=`bfn^Bg   h    - /   
+	I 	,	N 	6	P 	A	R 	,	R 	(	b 	,	g 		m 		x 		| 	  	  	  	  	 	 	  g  nameg  decode-response-body CR3KtjN   h    - /   
 0  
  3 #  #  	4 5#  #  #  #  #  #  	#  -44 5	$  "  
 5		4	>  G 
@@ -119,34 +119,34 @@ documentationk 0"   S   a   n   i   t   i   z   e   "       t   h   e       g  
 
 5$  4>  "  G  "   4&  		"  	45>  "  G  45
 &  
-"  	45$  !$  "  4>  "  G  
+"  	45&  !$  "  4>  "  G  
 D$  
 4
 5D4
 5$  "  4>  "  G  
 $  
 4
-5"  D      g  uri
+5"  D       g  uri
 	 g  body	 g  port		 g  method		 g  version		 g  keep-alive?		 g  headers		 g  decode-body?		 g  
 streaming?		 g  request			 g  request	
   g  body	  g  request	  g  response	
- g  body	i  g  filenamef  web/client.scm
+ g  body	h  g  filenamef  web/client.scm
  
 	# 		3 		= 		O 		j 		m 	  	  	(  	"  	  	  	  	  	  	  	
   	  	  	  	 	
- 	# 	-$ 	( 	
-. 	3 	I 	O 	
-R 	b 	c 	i 	q 	v 	 	 	 	 (	
+ 	# 	*' 	
+- 	2 	H 	N 	
+Q 	a 	b 	h 	p 	u 	 	 	 	 '	
 g  bodySg  portS	g  methodS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	g  requestS		 
-  g  nameg  request CR)f`b h   )  - /   	0  	 3 	#  #  	4 5#  #  #  #  $  "  #  #  $  4>  "  G  "    	
+  g  nameg  request CR3   h   )  - /   	0  	 3 	#  #  	4 5#  #  #  #  $  "  #  #  $  4>  "  G  "    	
 
 6      !      g  uri
 	  g  body	  g  port		  g  version		  g  keep-alive?		  g  
 extra-headers		  g  headers		  g  decode-body?		  g  
 streaming?		   	g  filenamef  web/client.scm
  
-	# 		3 		S 	/	Z 	A	t		u		y		{	 	 	 	 
+	# 		3 		S 	/	Z 	A	t		u		y		{	 	 	 	 
 	 
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  
 extra-headersS	g  headersS	g  decode-body?S	g  
@@ -176,28 +176,28 @@ documentation   k C   o   n   n   e   c   t       t   o       t   h   e       s
    
    R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   p   o   n   s   e       r   e   a   d       f   r   o   m       t   h   e       s   e   r   v   e   r   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e   
    b   o   d   y       a   s       a       s   t   r   i   n   g   ,       b   y   t   e   v   e   c   t   o   r   ,       #   f       v   a   l   u   e   ,       o   r       a   s       a       p   o   r   t       (   i   f       S   T   R   E   A   M   I   N   G   ?       i   s   
-   t   r   u   e   )   .    C*R)*f`b    h     - /   0   3 #  #  	4 5#  #  #  #  $  "  #  4>  "  G   	
+   t   r   u   e   )   .    C4R34    h     - /   0   3 #  #  	4 5#  #  #  #  $  "  #  4>  "  G   	
 
 6             g  uri
 	  g  body	  g  port		  g  version		  g  keep-alive?		  g  
 extra-headers		  g  headers		  g  decode-body?		   g  filenamef  web/client.scm
-
-	#		3		S	0	Z	B	f		j 		l!		q	 "	 	 
+#
+	#%		3&		S(	0	Z(	B	f+		j,		l-		q+	 .	 	 
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  
 extra-headersS	g  headersS	g  decode-body?S	   g  nameg  	http-get*g  
-documentationf  2Deprecated in favor of (http-get #:streaming? #t). C+R4 	     h(   i   ] C     a       g  dummy
-		# g  	http-verb		# g  method			# g  doc			#  		#	   C	      h(      ]	4 5$  @ 6             g  x
+documentationf  2Deprecated in favor of (http-get #:streaming? #t). C5R4	
+ 	 h(   i   ] C     a       g  dummy
+		# g  	http-verb		# g  method			# g  doc			#  		#	   C      h(      ]	4 5$  @ 6             g  x
 		" g  tmp		"  g  filenamef  web/client.scm
-&
+2
  		"  g  
-macro-typeg  syntax-rulesg  patternsg  	http-verbg  methodg  doc   C5R
-)f`b    h     - /   0   3 #  #  	4 5#  #  #  #  #   	
+macro-typeg  syntax-rulesg  patternsg  	http-verbg  methodg  doc   C5R3    h     - /   0   3 #  #  	4 5#  #  #  #  #   	
 
 6       g  uri
 		 g  body		 g  port			 g  version			 g  keep-alive?			 g  headers			 g  decode-body?			 g  
 streaming?			  g  filenamef  web/client.scm
-6
-	e7		6
+B
+	eC		B
  		
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	   g  nameg  	http-headg  
@@ -210,14 +210,13 @@ documentation k @F   e   t   c   h       m   e   s   s   a   g   e       h   e 
    
    R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d          #   f      .           R   e   s   p   o   n   s   e   s       t   o       H   E   A   D   
    r   e   q   u   e   s   t   s       d   o       n   o   t       h   a   v   e       a       b   o   d   y   .           T   h   e       s   e   c   o   n   d       v   a   l   u   e       i   s       o   n   l   y       r   e   t   u   r   n   e   d       s   o       t   h   a   t   
-   o   t   h   e   r       p   r   o   c   e   d   u   r   e   s       c   a   n       t   r   e   a   t       a   l   l       o   f       t   h   e       h   t   t   p   -   f   o   o       v   e   r   b   s       i   d   e   n   t   i   c   a   l   l   y   .    C,R
-)f`b        h   i  - /   0   3 #  #  	4 5#  #  #  #  #   	
+   o   t   h   e   r       p   r   o   c   e   d   u   r   e   s       c   a   n       t   r   e   a   t       a   l   l       o   f       t   h   e       h   t   t   p   -   f   o   o       v   e   r   b   s       i   d   e   n   t   i   c   a   l   l   y   .    C6R3        h   i  - /   0   3 #  #  	4 5#  #  #  #  #   	
 
 6 a      g  uri
 		 g  body		 g  port			 g  version			 g  keep-alive?			 g  headers			 g  decode-body?			 g  
 streaming?			  g  filenamef  web/client.scm
-C
-	eD		C
+O
+	eP		O
  		
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	   g  nameg  	http-postg  
@@ -227,14 +226,13 @@ documentation k P   o   s   t       d   a   t   a       t   o       t   h   e  
    m   e   t   h   o   d   .           S   e   e          h   t   t   p   -   g   e   t          f   o   r       f   u   l   l       d   o   c   u   m   e   n   t   a   t   i   o   n       o   n       t   h   e       v   a   r   i   o   u   s       k   e   y   w   o   r   d   
    a   r   g   u   m   e   n   t   s       t   h   a   t       a   r   e       a   c   c   e   p   t   e   d       b   y       t   h   i   s       f   u   n   c   t   i   o   n   .   
    
-   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C-R
-)f`b    h   ]  - /   0   3 #  #  	4 5#  #  #  #  #   	
+   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C7R3    h   ]  - /   0   3 #  #  	4 5#  #  #  #  #   	
 
 6 U      g  uri
 		 g  body		 g  port			 g  version			 g  keep-alive?			 g  headers			 g  decode-body?			 g  
 streaming?			  g  filenamef  web/client.scm
-M
-	eN		M
+Y
+	eZ		Y
  		
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	   g  nameg  http-putg  
@@ -244,14 +242,13 @@ documentation  k P   u   t       d   a   t   a       a   t       t   h   e     
    m   e   t   h   o   d   .           S   e   e          h   t   t   p   -   g   e   t          f   o   r       f   u   l   l       d   o   c   u   m   e   n   t   a   t   i   o   n       o   n       t   h   e       v   a   r   i   o   u   s       k   e   y   w   o   r   d   
    a   r   g   u   m   e   n   t   s       t   h   a   t       a   r   e       a   c   c   e   p   t   e   d       b   y       t   h   i   s       f   u   n   c   t   i   o   n   .   
    
-   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C.R
-)f`b        h     - /   0   3 #  #  	4 5#  #  #  #  #   	
+   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C8R3        h     - /   0   3 #  #  	4 5#  #  #  #  #   	
 
 6 }      g  uri
 		 g  body		 g  port			 g  version			 g  keep-alive?			 g  headers			 g  decode-body?			 g  
 streaming?			  g  filenamef  web/client.scm
-W
-	eX		W
+c
+	ed		c
  		
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	   g  nameg  http-deleteg  
@@ -261,14 +258,13 @@ documentation   k D   e   l   e   t   e       d   a   t   a       a   t       t
    m   e   t   h   o   d   .           S   e   e          h   t   t   p   -   g   e   t          f   o   r       f   u   l   l       d   o   c   u   m   e   n   t   a   t   i   o   n       o   n       t   h   e       v   a   r   i   o   u   s       k   e   y   w   o   r   d   
    a   r   g   u   m   e   n   t   s       t   h   a   t       a   r   e       a   c   c   e   p   t   e   d       b   y       t   h   i   s       f   u   n   c   t   i   o   n   .   
    
-   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C/R
-)f`b        h     - /   0   3 #  #  	4 5#  #  #  #  #   	
+   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C9R3        h     - /   0   3 #  #  	4 5#  #  #  #  #   	
 
 6       g  uri
 		 g  body		 g  port			 g  version			 g  keep-alive?			 g  headers			 g  decode-body?			 g  
 streaming?			  g  filenamef  web/client.scm
-a
-	eb		a
+m
+	en		m
  		
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	   g  nameg  
@@ -279,14 +275,13 @@ documentationk dS   e   n   d       a   n       H   T   T   P       "   T   R  
    m   e   t   h   o   d   .           S   e   e          h   t   t   p   -   g   e   t          f   o   r       f   u   l   l       d   o   c   u   m   e   n   t   a   t   i   o   n       o   n       t   h   e       v   a   r   i   o   u   s       k   e   y   w   o   r   d   
    a   r   g   u   m   e   n   t   s       t   h   a   t       a   r   e       a   c   c   e   p   t   e   d       b   y       t   h   i   s       f   u   n   c   t   i   o   n   .   
    
-   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C0R
-)f`b    h     - /   0   3 #  #  	4 5#  #  #  #  #   	
+   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C:R3    h     - /   0   3 #  #  	4 5#  #  #  #  #   	
 
 6       g  uri
 		 g  body		 g  port			 g  version			 g  keep-alive?			 g  headers			 g  decode-body?			 g  
 streaming?			  g  filenamef  web/client.scm
-k
-	el		k
+w
+	ex		w
  		
 g  bodySg  portS	g  versionS	g  keep-alive?S	g  headersS	g  decode-body?S	g  
 streaming?S	   g  nameg  http-optionsg  
@@ -297,21 +292,23 @@ documentation  k  Q   u   e   r   y       c   h   a   r   a   c   t   e   r   i
    m   e   t   h   o   d   .           S   e   e          h   t   t   p   -   g   e   t          f   o   r       f   u   l   l       d   o   c   u   m   e   n   t   a   t   i   o   n       o   n       t   h   e       v   a   r   i   o   u   s       k   e   y   w   o   r   d   
    a   r   g   u   m   e   n   t   s       t   h   a   t       a   r   e       a   c   c   e   p   t   e   d       b   y       t   h   i   s       f   u   n   c   t   i   o   n   .   
    
-   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C1RC         g  m
-		,  g  filenamef  web/client.scm		!
-F	4
-4	:
-	)	^
-	i
- ; 
+   R   e   t   u   r   n   s       t   w   o       v   a   l   u   e   s   :       t   h   e       r   e   s   u   l   t   i   n   g       r   e   s   p   o   n   s   e   ,       a   n   d       t   h   e       r   e   s   p   o   n   s   e       b   o   d   y   .    C;RC  %      g  m
+		, g  proxy
+	:	O  g  filenamef  web/client.scm		!
+	-	9		2	9		8	9	'	:	9		:	9		@	:	,	A	:		E	:		Q	9		T	8
+n	=
+	C
+	V	j
+	s
+ K 
 % 
 = 
-@
-L6
-S$C
-Z0M
-ahW
-ha
-ok
- 	o
+@#
+LB
+S,O
+Z8Y
+apc
+h$m
+ow
+ 	o
    C6 
\ No newline at end of file
